@@ -1,8 +1,9 @@
 import dataclasses
 import json
 import platform
+import warnings
 from random import randint
-from typing import Any, ClassVar, Dict
+from typing import Any, ClassVar
 
 try:
     from functools import cached_property
@@ -12,7 +13,12 @@ except ImportError:
 import pytest
 from dirty_equals import IsJson
 
-from pydantic_core import PydanticSerializationError, SchemaSerializer, SchemaValidator, core_schema
+from pydantic_core import (
+    PydanticSerializationError,
+    SchemaSerializer,
+    SchemaValidator,
+    core_schema,
+)
 
 from ..conftest import plain_repr
 
@@ -230,14 +236,17 @@ def test_model_wrong_warn():
     assert s.to_python(None, mode='json') is None
     assert s.to_json(None) == b'null'
 
-    with pytest.warns(UserWarning, match='Expected `MyModel` but got `int` - serialized value may.+'):
+    with pytest.warns(UserWarning, match='Expected `MyModel` but got `int` with value `123` - serialized value may.+'):
         assert s.to_python(123) == 123
-    with pytest.warns(UserWarning, match='Expected `MyModel` but got `int` - serialized value may.+'):
+    with pytest.warns(UserWarning, match='Expected `MyModel` but got `int` with value `123` - serialized value may.+'):
         assert s.to_python(123, mode='json') == 123
-    with pytest.warns(UserWarning, match='Expected `MyModel` but got `int` - serialized value may.+'):
+    with pytest.warns(UserWarning, match='Expected `MyModel` but got `int` with value `123` - serialized value may.+'):
         assert s.to_json(123) == b'123'
 
-    with pytest.warns(UserWarning, match='Expected `MyModel` but got `dict` - serialized value may.+'):
+    with pytest.warns(
+        UserWarning,
+        match="Expected `MyModel` but got `dict` with value `{'foo': 1, 'bar': b'more'}` - serialized value may.+",
+    ):
         assert s.to_python({'foo': 1, 'bar': b'more'}) == {'foo': 1, 'bar': b'more'}
 
 
@@ -398,7 +407,7 @@ def test_advanced_exclude_nested_lists(exclude, expected):
 
     # class SubModel(BaseModel):
     #     k: int
-    #     subsubs: List[SubSubModel]
+    #     subsubs: list[SubSubModel]
 
     sub_model_schema = core_schema.model_schema(
         type('SubModel', (), {}),
@@ -411,7 +420,7 @@ def test_advanced_exclude_nested_lists(exclude, expected):
     )
 
     # class Model(BaseModel):
-    #     subs: List[SubModel]
+    #     subs: list[SubModel]
 
     model_schema = core_schema.model_schema(
         BasicModel,
@@ -456,6 +465,58 @@ def test_function_plain_field_serializer_to_python():
     assert s.to_python(Model(x=1000)) == {'x': '1_000'}
 
 
+@pytest.mark.skipif(cached_property is None, reason='cached_property is not available')
+def test_field_serializer_cached_property():
+    @dataclasses.dataclass
+    class Model:
+        x: int
+        y: int
+
+        @cached_property
+        def x_formatted(self) -> str:
+            return f'{self.x:_}'
+
+        # This is a @computed_field
+        @cached_property
+        def y_formatted(self) -> str:
+            return f'{self.y:_}'
+
+        def ser_x(self, v: Any, _) -> str:
+            assert self.x == 1_000 == v
+            return self.x_formatted
+
+        def ser_y(self, v: Any, _) -> str:
+            assert self.y == 2_000 == v
+            return self.y_formatted
+
+    s = SchemaSerializer(
+        core_schema.model_schema(
+            Model,
+            core_schema.model_fields_schema(
+                {
+                    'x': core_schema.model_field(
+                        core_schema.int_schema(
+                            serialization=core_schema.plain_serializer_function_ser_schema(
+                                Model.ser_x, is_field_serializer=True, info_arg=True
+                            )
+                        )
+                    ),
+                    'y': core_schema.model_field(
+                        core_schema.int_schema(
+                            serialization=core_schema.plain_serializer_function_ser_schema(
+                                Model.ser_y, is_field_serializer=True, info_arg=True
+                            )
+                        )
+                    ),
+                },
+                computed_fields=[core_schema.computed_field('y_formatted', core_schema.str_schema())],
+            ),
+        )
+    )
+    assert s.to_python(Model(x=1000, y=2000)) == {'x': '1_000', 'y': '2_000', 'y_formatted': '2_000'}
+    assert s.to_json(Model(x=1000, y=2000)) == b'{"x":"1_000","y":"2_000","y_formatted":"2_000"}'
+
+
 def test_function_wrap_field_serializer_to_python():
     @dataclasses.dataclass
     class Model:
@@ -513,6 +574,42 @@ def test_function_plain_field_serializer_to_json():
     assert json.loads(s.to_json(Model(x=1000))) == {'x': '1_000'}
 
 
+def test_function_plain_field_serializer_with_computed_field():
+    @dataclasses.dataclass
+    class Model:
+        x: int
+
+        @property
+        def computed_field_x(self) -> int:
+            return self.x + 200
+
+        def ser_func(self, v: Any, info: core_schema.FieldSerializationInfo) -> str:
+            return info.field_name + '_' + str(v * 2)
+
+    field_str_with_field_serializer = core_schema.str_schema(
+        serialization=core_schema.plain_serializer_function_ser_schema(
+            Model.ser_func,
+            is_field_serializer=True,
+            info_arg=True,
+            return_schema=core_schema.any_schema(),
+        )
+    )
+
+    s = SchemaSerializer(
+        core_schema.model_schema(
+            Model,
+            core_schema.model_fields_schema(
+                {'x': core_schema.model_field(field_str_with_field_serializer)},
+                computed_fields=[
+                    core_schema.computed_field('computed_field_x', field_str_with_field_serializer),
+                ],
+            ),
+        )
+    )
+    assert json.loads(s.to_json(Model(x=1000))) == {'x': 'x_2000', 'computed_field_x': 'computed_field_x_2400'}
+    assert s.to_python(Model(x=2000)) == {'x': 'x_4000', 'computed_field_x': 'computed_field_x_4400'}
+
+
 def test_function_wrap_field_serializer_to_json():
     @dataclasses.dataclass
     class Model:
@@ -650,6 +747,51 @@ def test_computed_field_exclude_none():
     assert s.to_json(Model(3, 4), exclude_none=True) == b'{"width":3,"height":4,"Area":12}'
 
 
+def test_computed_field_exclude_none_different_order():
+    # verify that order of computed fields doesn't matter
+    # issue originally reported via: https://github.com/pydantic/pydantic/issues/8691
+
+    @dataclasses.dataclass
+    class Model:
+        width: int
+        height: int
+
+        @property
+        def volume(self) -> None:
+            return None
+
+        @property
+        def area(self) -> int:
+            return self.width * self.height
+
+    s = SchemaSerializer(
+        core_schema.model_schema(
+            Model,
+            core_schema.model_fields_schema(
+                {
+                    'width': core_schema.model_field(core_schema.int_schema()),
+                    'height': core_schema.model_field(core_schema.int_schema()),
+                },
+                computed_fields=[
+                    core_schema.computed_field('volume', core_schema.int_schema()),
+                    core_schema.computed_field('area', core_schema.int_schema(), alias='Area'),
+                ],
+            ),
+        )
+    )
+    assert s.to_python(Model(3, 4), exclude_none=False) == {'width': 3, 'height': 4, 'Area': 12, 'volume': None}
+    assert s.to_python(Model(3, 4), exclude_none=True) == {'width': 3, 'height': 4, 'Area': 12}
+    assert s.to_python(Model(3, 4), mode='json', exclude_none=False) == {
+        'width': 3,
+        'height': 4,
+        'Area': 12,
+        'volume': None,
+    }
+    assert s.to_python(Model(3, 4), mode='json', exclude_none=True) == {'width': 3, 'height': 4, 'Area': 12}
+    assert s.to_json(Model(3, 4), exclude_none=False) == b'{"width":3,"height":4,"volume":null,"Area":12}'
+    assert s.to_json(Model(3, 4), exclude_none=True) == b'{"width":3,"height":4,"Area":12}'
+
+
 @pytest.mark.skipif(cached_property is None, reason='cached_property is not available')
 def test_cached_property_alias():
     @dataclasses.dataclass
@@ -925,7 +1067,7 @@ def test_extra_config_nested_model():
 def test_extra_custom_serializer():
     class Model:
         __slots__ = ('__pydantic_extra__', '__dict__')
-        __pydantic_extra__: Dict[str, Any]
+        __pydantic_extra__: dict[str, Any]
 
     schema = core_schema.model_schema(
         Model,
@@ -944,3 +1086,69 @@ def test_extra_custom_serializer():
     m.__pydantic_extra__ = {'extra': 'extra'}
 
     assert s.to_python(m) == {'extra': 'extra bam!'}
+
+
+def test_no_warn_on_exclude() -> None:
+    with warnings.catch_warnings():
+        warnings.simplefilter('error')
+
+        s = SchemaSerializer(
+            core_schema.model_schema(
+                BasicModel,
+                core_schema.model_fields_schema(
+                    {
+                        'a': core_schema.model_field(core_schema.int_schema()),
+                        'b': core_schema.model_field(core_schema.int_schema()),
+                    }
+                ),
+            )
+        )
+
+        value = BasicModel(a=0, b=1)
+        assert s.to_python(value, exclude={'b'}) == {'a': 0}
+        assert s.to_python(value, mode='json', exclude={'b'}) == {'a': 0}
+
+
+def test_warn_on_missing_field() -> None:
+    class AModel(BasicModel): ...
+
+    class BModel(BasicModel): ...
+
+    s = SchemaSerializer(
+        core_schema.model_schema(
+            BasicModel,
+            core_schema.model_fields_schema(
+                {
+                    'root': core_schema.model_field(
+                        core_schema.tagged_union_schema(
+                            choices={
+                                'a': core_schema.model_schema(
+                                    AModel,
+                                    core_schema.model_fields_schema(
+                                        {
+                                            'type': core_schema.model_field(core_schema.literal_schema(['a'])),
+                                            'a': core_schema.model_field(core_schema.int_schema()),
+                                        }
+                                    ),
+                                ),
+                                'b': core_schema.model_schema(
+                                    BModel,
+                                    core_schema.model_fields_schema(
+                                        {
+                                            'type': core_schema.model_field(core_schema.literal_schema(['b'])),
+                                            'b': core_schema.model_field(core_schema.int_schema()),
+                                        }
+                                    ),
+                                ),
+                            },
+                            discriminator='type',
+                        )
+                    ),
+                }
+            ),
+        )
+    )
+
+    with pytest.warns(UserWarning, match='Expected 2 fields but got 1 for type `.*AModel` with value `.*`.+'):
+        value = BasicModel(root=AModel(type='a'))
+        s.to_python(value)
