@@ -1,6 +1,7 @@
 #[cfg(test)]
 mod tests {
-    use _pydantic_core::{SchemaSerializer, SchemaValidator};
+    use _pydantic_core::{SchemaSerializer, SchemaValidator, WarningsArg};
+    use pyo3::ffi::c_str; // can switch to c"" literals on MSRV >= 1.77
     use pyo3::prelude::*;
     use pyo3::types::PyDict;
 
@@ -22,7 +23,8 @@ mod tests {
             //         'type': 'function-wrap',
             //         'function': lambda: None,
             //     },
-            let code = r"{
+            let code = c_str!(
+                r"{
                 'type': 'definitions',
                 'schema': {'type': 'definition-ref', 'schema_ref': 'C-ref'},
                 'definitions': [
@@ -44,8 +46,9 @@ mod tests {
                         },
                     },
                 ]
-            }";
-            let schema: Bound<'_, PyDict> = py.eval_bound(code, None, None).unwrap().extract().unwrap();
+            }"
+            );
+            let schema: Bound<'_, PyDict> = py.eval(code, None, None).unwrap().extract().unwrap();
             SchemaSerializer::py_new(schema, None).unwrap();
         });
     }
@@ -53,7 +56,8 @@ mod tests {
     #[test]
     fn test_serialize_computed_fields() {
         Python::with_gil(|py| {
-            let code = r#"
+            let code = c_str!(
+                r#"
 class A:
     @property
     def b(self) -> str:
@@ -72,9 +76,10 @@ schema = {
     "type": "model",
 }
 a = A()
-            "#;
-            let locals = PyDict::new_bound(py);
-            py.run_bound(code, None, Some(&locals)).unwrap();
+            "#
+            );
+            let locals = PyDict::new(py);
+            py.run(code, None, Some(&locals)).unwrap();
             let a = locals.get_item("a").unwrap().unwrap();
             let schema = locals
                 .get_item("schema")
@@ -85,7 +90,20 @@ a = A()
             let serialized = SchemaSerializer::py_new(schema, None)
                 .unwrap()
                 .to_json(
-                    py, &a, None, None, None, true, false, false, false, false, true, None, false, None,
+                    py,
+                    &a,
+                    None,
+                    None,
+                    None,
+                    true,
+                    false,
+                    false,
+                    false,
+                    false,
+                    WarningsArg::Bool(true),
+                    None,
+                    false,
+                    None,
                 )
                 .unwrap();
             let serialized: &[u8] = serialized.extract(py).unwrap();
@@ -96,7 +114,8 @@ a = A()
     #[test]
     fn test_literal_schema() {
         Python::with_gil(|py| {
-            let code = r#"
+            let code = c_str!(
+                r#"
 schema = {
     "type": "dict",
     "keys_schema": {
@@ -109,14 +128,15 @@ schema = {
     "strict": False,
 }
 json_input = '{"a": "something"}'
-            "#;
-            let locals = PyDict::new_bound(py);
-            py.run_bound(code, None, Some(&locals)).unwrap();
+            "#
+            );
+            let locals = PyDict::new(py);
+            py.run(code, None, Some(&locals)).unwrap();
             let schema = locals.get_item("schema").unwrap().unwrap();
             let json_input = locals.get_item("json_input").unwrap().unwrap();
             let binding = SchemaValidator::py_new(py, &schema, None)
                 .unwrap()
-                .validate_json(py, &json_input, None, None, None)
+                .validate_json(py, &json_input, None, None, None, false.into())
                 .unwrap();
             let validation_result: Bound<'_, PyAny> = binding.extract(py).unwrap();
             let repr = format!("{}", validation_result.repr().unwrap());
@@ -127,7 +147,8 @@ json_input = '{"a": "something"}'
     #[test]
     fn test_segfault_for_recursive_schemas() {
         Python::with_gil(|py| {
-            let code = r"
+            let code = c_str!(
+                r"
 schema = {
     'type': 'definitions',
     'schema': {
@@ -162,9 +183,10 @@ schema = {
 }
 dump_json_input_1 = 1
 dump_json_input_2 = {'a': 'something'}
-            ";
-            let locals = PyDict::new_bound(py);
-            py.run_bound(code, None, Some(&locals)).unwrap();
+            "
+            );
+            let locals = PyDict::new(py);
+            py.run(code, None, Some(&locals)).unwrap();
             let schema = locals
                 .get_item("schema")
                 .unwrap()
@@ -186,7 +208,7 @@ dump_json_input_2 = {'a': 'something'}
                     false,
                     false,
                     false,
-                    false,
+                    WarningsArg::Bool(false),
                     None,
                     false,
                     None,
@@ -207,7 +229,7 @@ dump_json_input_2 = {'a': 'something'}
                     false,
                     false,
                     false,
-                    false,
+                    WarningsArg::Bool(false),
                     None,
                     false,
                     None,
