@@ -2,18 +2,12 @@ import copy
 import json
 import platform
 import re
+import zoneinfo
 from datetime import date, datetime, time, timedelta, timezone, tzinfo
 from decimal import Decimal
-from typing import Dict
 
 import pytest
 
-try:
-    import zoneinfo
-except ImportError:
-    # TODO: can remove this once we drop support for python 3.8
-    from backports import zoneinfo
-
 from pydantic_core import SchemaError, SchemaValidator, ValidationError, core_schema, validate_core_schema
 
 from ..conftest import Err, PyAndJson
@@ -53,6 +47,14 @@ from ..conftest import Err, PyAndJson
                 'Input should be a valid datetime or date, day value is outside expected range [type=datetime_from_date_parsing,'
             ),
         ),
+        (
+            '0001-01-01T00:00:00.000000Z',
+            datetime(1, 1, 1, tzinfo=timezone.utc),
+        ),
+        (
+            '0000-12-31T23:59:59.999999Z',
+            Err('Input should be a valid datetime, year 0 is out of range [type=datetime_parsing,'),
+        ),
     ],
 )
 def test_datetime(input_value, expected):
@@ -500,7 +502,7 @@ def test_tz_constraint_wrong():
 
 def test_tz_hash() -> None:
     v = SchemaValidator(core_schema.datetime_schema())
-    lookup: Dict[datetime, str] = {}
+    lookup: dict[datetime, str] = {}
     for day in range(1, 10):
         input_str = f'2022-06-{day:02}T12:13:14-12:15'
         validated = v.validate_python(input_str)
