@@ -1,7 +1,7 @@
 import datetime
 import platform
 from dataclasses import dataclass
-from typing import List, Optional
+from typing import Optional
 
 import pytest
 from dirty_equals import AnyThing, HasAttributes, IsList, IsPartialDict, IsStr, IsTuple
@@ -145,7 +145,7 @@ def test_multiple_intertwined():
         height: int
         class Bar:
             width: int
-            bars: List['Bar']
+            bars: list['Bar']
             foo: Optional['Foo']
         bar = Bar
     """
@@ -279,7 +279,7 @@ def test_outside_parent():
                 }
             ),
             [
-                core_schema.tuple_positional_schema(
+                core_schema.tuple_schema(
                     [core_schema.int_schema(), core_schema.int_schema(), core_schema.str_schema()], ref='tuple-iis'
                 )
             ],
@@ -426,7 +426,7 @@ def multiple_tuple_schema() -> SchemaValidator:
                 }
             ),
             [
-                core_schema.tuple_positional_schema(
+                core_schema.tuple_schema(
                     [
                         core_schema.int_schema(),
                         core_schema.nullable_schema(core_schema.definition_reference_schema('t')),
@@ -522,7 +522,7 @@ def test_definition_wrap():
             [
                 core_schema.with_info_wrap_validator_function(
                     wrap_func,
-                    core_schema.tuple_positional_schema(
+                    core_schema.tuple_schema(
                         [
                             core_schema.int_schema(),
                             core_schema.nullable_schema(core_schema.definition_reference_schema('wrapper')),
@@ -852,7 +852,7 @@ def test_unsorted_definitions_schema() -> None:
 def test_validate_assignment(pydantic_version) -> None:
     @dataclass
     class Model:
-        x: List['Model']
+        x: list['Model']
 
     schema = core_schema.definitions_schema(
         core_schema.definition_reference_schema('model'),
