@@ -1,7 +1,7 @@
 import json
 import re
 from decimal import Decimal
-from typing import Any, Dict
+from typing import Any
 
 import pytest
 from dirty_equals import IsStr
@@ -20,13 +20,58 @@ i64_max = 9_223_372_036_854_775_807
         (True, 1),
         (0, 0),
         ('0', 0),
+        ('00', 0),
+        ('000', 0),
+        ('0_000', 0),
+        ('+0', 0),
+        ('+00', 0),
+        ('+000', 0),
+        ('+0_000', 0),
         (1, 1),
         ('  1  ', 1),
+        ('-1', -1),
+        ('-1.0', -1),
         (42, 42),
         ('42', 42),
         (42.0, 42),
+        ('0.0', 0),
+        ('00.0', 0),
+        ('00.00', 0),
         ('42.0', 42),
+        ('42.00', 42),
+        ('042', 42),
+        ('01', 1),
+        ('09', 9),
+        ('00_', Err('Input should be a valid integer, unable to parse string as an integer')),
+        # next character after 9 is not valid
+        ('0:', Err('Input should be a valid integer, unable to parse string as an integer')),
+        ('+4_2', 42),
+        ('+0_42', 42),
+        ('+4_2.0', 42),
+        ('+04_2.0', 42),
+        ('++4_2', Err('Input should be a valid integer, unable to parse string as an integer')),
+        ('-+1', Err('Input should be a valid integer, unable to parse string as an integer')),
+        ('+-1', Err('Input should be a valid integer, unable to parse string as an integer')),
+        ('--0001', Err('Input should be a valid integer, unable to parse string as an integer')),
+        ('-+0001', Err('Input should be a valid integer, unable to parse string as an integer')),
+        ('-0-001', Err('Input should be a valid integer, unable to parse string as an integer')),
+        ('-0+001', Err('Input should be a valid integer, unable to parse string as an integer')),
+        ('-00001', -1),
+        ('-00042_000', -42000),
+        ('4_2', 42),
+        ('0_42', 42),
+        ('4_2.0', 42),
+        ('04_2.0', 42),
+        ('  04_2.0 ', 42),
+        ('  0_42.0 ', 42),
+        ('  _042.0 ', Err('Input should be a valid integer, unable to parse string as an integer')),
+        ('42_', Err('Input should be a valid integer, unable to parse string as an integer')),
+        ('42_.0', Err('Input should be a valid integer, unable to parse string as an integer')),
+        ('000001', 1),
         ('123456789.0', 123_456_789),
+        (' ', Err('Input should be a valid integer, unable to parse string as an integer')),
+        ('1.', Err('Input should be a valid integer, unable to parse string as an integer')),
+        ('42.', Err('Input should be a valid integer, unable to parse string as an integer')),
         ('123456789123456.00001', Err('Input should be a valid integer, unable to parse string as an integer')),
         (int(1e10), int(1e10)),
         (i64_max, i64_max),
@@ -266,10 +311,7 @@ def test_int_strict(py_and_json: PyAndJson, input_value, expected):
         (
             {'ge': 0},
             -1,
-            Err(
-                'Input should be greater than or equal to 0 '
-                '[type=greater_than_equal, input_value=-1, input_type=int]'
-            ),
+            Err('Input should be greater than or equal to 0 [type=greater_than_equal, input_value=-1, input_type=int]'),
         ),
         ({'gt': 0}, 1, 1),
         ({'gt': 0}, 0, Err('Input should be greater than 0 [type=greater_than, input_value=0, input_type=int]')),
@@ -283,7 +325,7 @@ def test_int_strict(py_and_json: PyAndJson, input_value, expected):
     ],
     ids=repr,
 )
-def test_int_kwargs(py_and_json: PyAndJson, kwargs: Dict[str, Any], input_value, expected):
+def test_int_kwargs(py_and_json: PyAndJson, kwargs: dict[str, Any], input_value, expected):
     v = py_and_json({'type': 'int', **kwargs})
     if isinstance(expected, Err):
         with pytest.raises(ValidationError, match=re.escape(expected.message)) as exc_info:
