@@ -2,7 +2,7 @@ import re
 import sys
 from copy import deepcopy
 from decimal import Decimal
-from typing import Any, Callable, Dict, List, Set, Tuple
+from typing import Any, Callable
 
 import pytest
 from dirty_equals import HasRepr, IsInstance
@@ -206,8 +206,8 @@ def test_model_class_root_validator_wrap():
             self.__dict__.update(kwargs)
 
     def f(
-        input_value: Dict[str, Any],
-        validator: Callable[[Dict[str, Any]], Dict[str, Any]],
+        input_value: dict[str, Any],
+        validator: Callable[[dict[str, Any]], dict[str, Any]],
         info: core_schema.ValidationInfo,
     ):
         assert input_value['field_a'] == 123
@@ -244,7 +244,7 @@ def test_model_class_root_validator_before():
         def __init__(self, **kwargs: Any) -> None:
             self.__dict__.update(kwargs)
 
-    def f(input_value: Dict[str, Any], info: core_schema.ValidationInfo):
+    def f(input_value: dict[str, Any], info: core_schema.ValidationInfo):
         assert input_value['field_a'] == 123
         return input_value
 
@@ -278,7 +278,7 @@ def test_model_class_root_validator_after():
         def __init__(self, **kwargs: Any) -> None:
             self.__dict__.update(kwargs)
 
-    def f(input_value_and_fields_set: Tuple[Dict[str, Any], Set[str]]):
+    def f(input_value_and_fields_set: tuple[dict[str, Any], set[str]]):
         input_value, _, _ = input_value_and_fields_set
         assert input_value['field_a'] == 123
         return input_value_and_fields_set
@@ -1036,17 +1036,17 @@ def test_validate_assignment():
             self.__pydantic_extra__ = None
 
     v = SchemaValidator(
-        {
-            'type': 'model',
-            'cls': MyModel,
-            'schema': {
-                'type': 'model-fields',
-                'fields': {
-                    'field_a': {'type': 'model-field', 'schema': {'type': 'str'}},
-                    'field_b': {'type': 'model-field', 'schema': {'type': 'int'}},
+        core_schema.model_schema(
+            MyModel,
+            core_schema.model_fields_schema(
+                {
+                    'field_a': core_schema.model_field(core_schema.str_schema()),
+                    'field_b': core_schema.model_field(core_schema.int_schema()),
                 },
-            },
-        }
+                extra_behavior='allow',
+            ),
+            extra_behavior='allow',
+        )
     )
 
     m = MyModel()
@@ -1063,6 +1063,12 @@ def test_validate_assignment():
     v.validate_assignment(m, 'field_b', '322', from_attributes=True)
     assert m.field_b == 322
 
+    # try deleting a field
+    del m.field_b
+    # assignment to `field_a` should not care about `field_b` missing
+    v.validate_assignment(m, 'field_a', 'hello world', from_attributes=True)
+    assert m.field_a == 'hello world'
+
 
 def test_validate_assignment_function():
     class MyModel:
@@ -1072,7 +1078,7 @@ def test_validate_assignment_function():
         field_b: int
         field_c: int
 
-    calls: List[Any] = []
+    calls: list[Any] = []
 
     def func(x, info):
         calls.append(str(info))
@@ -1208,7 +1214,7 @@ def test_validate_assignment_model_validator_function(function_schema: Any, call
     class Model:
         __slots__ = '__dict__', '__pydantic_fields_set__', '__pydantic_extra__', '__pydantic_private__'
 
-    calls: List[Any] = []
+    calls: list[Any] = []
 
     def f(values_or_values_and_fields_set: Any, *args: Any) -> Any:
         if len(args) == 2:
