@@ -1,14 +1,15 @@
 import math
 import re
 import sys
+from collections.abc import Mapping
 from dataclasses import dataclass
 from datetime import datetime
-from typing import Any, Dict, List, Mapping, Union
+from typing import Any, Union
 
 import pytest
 from dirty_equals import FunctionCheck, HasRepr, IsStr
 
-from pydantic_core import CoreConfig, SchemaError, SchemaValidator, ValidationError, core_schema
+from pydantic_core import CoreConfig, SchemaError, SchemaValidator, ValidationError, core_schema, validate_core_schema
 
 from ..conftest import Err, PyAndJson
 
@@ -32,8 +33,8 @@ class Map(Mapping):
     def __len__(self) -> int:
         return len(self._d)
 
-    def __getitem__(self, __k):
-        return self._d[__k]
+    def __getitem__(self, k, /):
+        return self._d[k]
 
     def __repr__(self):
         return 'Map({})'.format(', '.join(f'{k}={v!r}' for k, v in self._d.items()))
@@ -210,9 +211,9 @@ def test_forbid_extra():
 
 
 def test_allow_extra_invalid():
-    with pytest.raises(SchemaError, match='extra_validator can only be used if extra_behavior=allow'):
+    with pytest.raises(SchemaError, match='extras_schema can only be used if extra_behavior=allow'):
         SchemaValidator(
-            {'type': 'model-fields', 'fields': {}, 'extra_validator': {'type': 'int'}, 'extra_behavior': 'ignore'}
+            {'type': 'model-fields', 'fields': {}, 'extras_schema': {'type': 'int'}, 'extra_behavior': 'ignore'}
         )
 
 
@@ -280,7 +281,7 @@ def test_validate_assignment_functions():
                     'type': 'model-field',
                     'schema': {
                         'type': 'function-after',
-                        'function': {'type': 'general', 'function': func_a},
+                        'function': {'type': 'with-info', 'function': func_a},
                         'schema': {'type': 'str'},
                     },
                 },
@@ -288,7 +289,7 @@ def test_validate_assignment_functions():
                     'type': 'model-field',
                     'schema': {
                         'type': 'function-after',
-                        'function': {'type': 'general', 'function': func_b},
+                        'function': {'type': 'with-info', 'function': func_b},
                         'schema': {'type': 'int'},
                     },
                 },
@@ -358,7 +359,7 @@ def test_validate_assignment_allow_extra_validate():
         {
             'type': 'model-fields',
             'fields': {'field_a': {'type': 'model-field', 'schema': {'type': 'str'}}},
-            'extra_validator': {'type': 'int'},
+            'extras_schema': {'type': 'int'},
             'extra_behavior': 'allow',
         }
     )
@@ -430,7 +431,7 @@ def test_json_error():
 
 def test_missing_schema_key():
     with pytest.raises(SchemaError, match='model-fields.fields.x.schema\n  Field required'):
-        SchemaValidator({'type': 'model-fields', 'fields': {'x': {'type': 'str'}}})
+        validate_core_schema({'type': 'model-fields', 'fields': {'x': {'type': 'str'}}})
 
 
 def test_fields_required_by_default():
@@ -734,10 +735,12 @@ def test_paths_allow_by_name(py_and_json: PyAndJson, input_value):
 def test_alias_build_error(alias_schema, error):
     with pytest.raises(SchemaError, match=error):
         SchemaValidator(
-            {
-                'type': 'model-fields',
-                'fields': {'field_a': {'type': 'model-field', 'schema': {'type': 'int'}, **alias_schema}},
-            }
+            validate_core_schema(
+                {
+                    'type': 'model-fields',
+                    'fields': {'field_a': {'type': 'model-field', 'schema': {'type': 'int'}, **alias_schema}},
+                }
+            )
         )
 
 
@@ -1180,15 +1183,15 @@ def test_from_attributes_extra_ignore_no_attributes_accessed() -> None:
         }
     )
 
-    accessed: List[str] = []
+    accessed: list[str] = []
 
     class Source:
         a = 1
         b = 2
 
-        def __getattribute__(self, __name: str) -> Any:
-            accessed.append(__name)
-            return super().__getattribute__(__name)
+        def __getattribute__(self, name: str, /) -> Any:
+            accessed.append(name)
+            return super().__getattribute__(name)
 
     assert v.validate_python(Source()) == ({'a': 1}, None, {'a'})
     assert 'a' in accessed and 'b' not in accessed
@@ -1491,7 +1494,7 @@ def test_bad_default_factory(default_factory, error_message):
 class TestOnError:
     def test_on_error_bad_name(self):
         with pytest.raises(SchemaError, match="Input should be 'raise', 'omit' or 'default'"):
-            SchemaValidator(
+            validate_core_schema(
                 {
                     'type': 'model-fields',
                     'fields': {
@@ -1606,7 +1609,7 @@ class TestOnError:
                             'on_error': 'raise',
                             'schema': {
                                 'type': 'function-wrap',
-                                'function': {'type': 'general', 'function': wrap_function},
+                                'function': {'type': 'with-info', 'function': wrap_function},
                                 'schema': {'type': 'str'},
                             },
                         },
@@ -1659,19 +1662,19 @@ def test_frozen_field():
     ],
 )
 @pytest.mark.parametrize(
-    'extra_validator_kw, expected_extra_value',
-    [({}, '123'), ({'extra_validator': None}, '123'), ({'extra_validator': core_schema.int_schema()}, 123)],
-    ids=['extra_validator=unset', 'extra_validator=None', 'extra_validator=int'],
+    'extras_schema_kw, expected_extra_value',
+    [({}, '123'), ({'extras_schema': None}, '123'), ({'extras_schema': core_schema.int_schema()}, 123)],
+    ids=['extras_schema=unset', 'extras_schema=None', 'extras_schema=int'],
 )
 def test_extra_behavior_allow(
     config: Union[core_schema.CoreConfig, None],
-    schema_extra_behavior_kw: Dict[str, Any],
-    extra_validator_kw: Dict[str, Any],
+    schema_extra_behavior_kw: dict[str, Any],
+    extras_schema_kw: dict[str, Any],
     expected_extra_value: Any,
 ):
     v = SchemaValidator(
         core_schema.model_fields_schema(
-            {'f': core_schema.model_field(core_schema.str_schema())}, **schema_extra_behavior_kw, **extra_validator_kw
+            {'f': core_schema.model_field(core_schema.str_schema())}, **schema_extra_behavior_kw, **extras_schema_kw
         ),
         config=config,
     )
@@ -1700,7 +1703,7 @@ def test_extra_behavior_allow(
         (core_schema.CoreConfig(extra_fields_behavior='allow'), {'extra_behavior': 'forbid'}),
     ],
 )
-def test_extra_behavior_forbid(config: Union[core_schema.CoreConfig, None], schema_extra_behavior_kw: Dict[str, Any]):
+def test_extra_behavior_forbid(config: Union[core_schema.CoreConfig, None], schema_extra_behavior_kw: dict[str, Any]):
     v = SchemaValidator(
         core_schema.model_fields_schema(
             {'f': core_schema.model_field(core_schema.str_schema())}, **schema_extra_behavior_kw
@@ -1747,7 +1750,7 @@ def test_extra_behavior_forbid(config: Union[core_schema.CoreConfig, None], sche
         (None, {'extra_behavior': None}),
     ],
 )
-def test_extra_behavior_ignore(config: Union[core_schema.CoreConfig, None], schema_extra_behavior_kw: Dict[str, Any]):
+def test_extra_behavior_ignore(config: Union[core_schema.CoreConfig, None], schema_extra_behavior_kw: dict[str, Any]):
     v = SchemaValidator(
         core_schema.model_fields_schema(
             {'f': core_schema.model_field(core_schema.str_schema())}, **schema_extra_behavior_kw
